/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86_Option.c,v 3.1 1996/12/23 06:43:45 dawes Exp $ */
/*
 *
 * The purpose of this file is to initialise the xf86_OptionTab[] which
 * is setup in xf86_Option.h
 */
/* $XConsortium: xf86_Option.c /main/3 1996/02/21 17:39:22 kaleb $ */

#define INIT_OPTIONS
#define _NO_XF86_PROTOTYPES

#include "X.h"
#include "os.h"
#include "xf86.h"
