/* $XFree86: xc/programs/Xserver/hw/xfree86/vga256/drivers/rendition/hwcursor.h,v 1.1.2.2 1998/08/07 06:40:20 hohndel Exp $ */
/*
 * file hwcursor.h
 */

#ifndef __HWCURSOR_H__



/*
 * functions prototypes
 */

void RENDITIONHwCursorInit(void);



#endif /* #ifndef __HWCURSOR_H__ */

/*
 * end of file hwcursor.h
 */

